HANDLE WINAPI GetStdHandle(DWORD nStdHandle);
BOOL WINAPI WriteConsoleA(HANDLE hConsoleOutput, const VOID *lpBuffer, DWORD nNumberOfCharsToWrite, LPDWORD lpNumberOfCharsWritten, LPVOID lpReserved);
LPTSTR WINAPI GetCommandLineA(void);
VOID WINAPI ExitProcess(UINT uExitCode);

HANDLE stdin;
HANDLE stdout;
HANDLE stderr;

const char dec[10] = {'0','1','2','3','4','5','6','7','8','9'};
const char hex[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

void InitStdio() {
	stdin = GetStdHandle(-10);
	stdout = GetStdHandle(-11);
	stderr = GetStdHandle(-12);
};
int GetTextLength(const char *lpText) {
	int txtLen = 0;
	while(lpText[txtLen] != '\0') { txtLen++; };
	return txtLen;
};
BOOL ConsoleWriteChar(char pChar) {
	return WriteFile(stdout, (LPVOID) &pChar, 1, NULL, 0);
};
BOOL ConsoleWrite(const char *lpText) {
	DWORD textLength;
	textLength = GetTextLength(lpText);
	return WriteFile(stdout, (LPVOID) lpText, textLength, &textLength, 0);
};
BOOL ConsoleWriteOnly(const char *lpText) {
	DWORD textLength;
	textLength = GetTextLength(lpText);
	return WriteConsoleA(stdout, lpText, textLength, &textLength, 0);
};
BOOL ConsoleWriteError(const char *lpText) {
	DWORD textLength;
	textLength = GetTextLength(lpText);
	return WriteFile(stderr, (LPVOID) lpText, textLength, &textLength, 0);
};

void ConsoleWriteLine(const char *lpText) {
	ConsoleWrite(lpText);
	ConsoleWrite("\r\n\0");
};
void ConsoleWriteLineOnly(const char *lpText) {
	ConsoleWriteOnly(lpText);
	ConsoleWriteOnly("\r\n\0");
};
void ConsoleWriteLineError(const char *lpText) {
	ConsoleWriteError(lpText);
	ConsoleWriteError("\r\n\0");
};
void IntToHex(int val, char *lpBuffer) {
	int bitlen = 8;
	int pos = 0;
	while(pos < bitlen) {
		lpBuffer[bitlen - (pos + 1)] = hex[(val >> (4 * pos)) & 0x0F];
		pos++;
	};
	lpBuffer[bitlen] = '\0';
};
int StringToInt(const char *lpText) {
	int txtlen = GetTextLength(lpText);
	int nval = -1;
	int rval = 0;
	int pos = 0;
	int chk = 0;
	for(pos = 0; pos < txtlen; pos++) {
		nval = -1;
		for(chk = 0; chk < 10; chk++) {
			if(lpText[pos] == dec[chk]) {
				nval = chk;
				chk = 10;
			};
		};
		if(nval == -1) { return -1; };
		rval *= 10;
		rval += nval;
	};
	return rval;
};

void ConsoleWriteHex(int number) {
	char* lpt = (char*) MemAlloc(9);
	IntToHex(number, lpt);
	ConsoleWrite(lpt);
	MemFree(lpt);
};
int GetCommandLine(LPTSTR lpArgs[]) {
	int number = 0;
	BOOL inquot = FALSE;
	int pos = 0;
	LPTSTR rawline = GetCommandLineA();
	number = 0;
	while(rawline[pos] != '\0') {
		lpArgs[number] = (LPTSTR) (rawline + pos);
cont:
		while((rawline[pos] != ' ') && (rawline[pos] != '\0')) { 
			if(rawline[pos] == '"') {
				if(inquot == FALSE) {
					inquot = TRUE;
				} else {
					inquot = FALSE;
				};
			};
			pos++; 
		};
		if(rawline[pos] != '\0') {
			if((number == 0) && (rawline[pos + 1] == ' ')) { pos++; };
			pos++;
			if(inquot == TRUE) { goto cont; };
			rawline[pos-1] = '\0';
		};
		number++;
	};
	return number;
};
void RemoveQuotes(LPTSTR string) {
	int vpos = 0;
	int rpos = 0;
	while(string[vpos] != '\0') {
		while(string[vpos] == '"') { vpos++; };
		string[rpos] = string[vpos];
		rpos++;
		if(string[vpos] != '\0') { vpos++; };
	};
	string[rpos] = '\0';
};
LPTSTR AppendString(LPTSTR base, LPTSTR toappend) {
	int inlen = GetTextLength(base) - 1;
	int aplen = GetTextLength(toappend) - 1;
	int idx = 0;
	LPTSTR outstring = (LPTSTR) MemAlloc(inlen + aplen + 2);
	for(idx = 0; idx <= inlen; idx++) {
		outstring[idx] = base[idx];
	};
	for(idx = 0; idx <= aplen; idx++) {
		outstring[idx + inlen + 1] = toappend[idx];
	};
	outstring[inlen + aplen + 2] = '\0';
	return outstring;
};