//TODO: Replace division operations with lookup tables or bitshifts (division can be found in SSE and the rangecoder)
//TODO: Crashes when decoding a file with a filename 20 characters in length without specifying an output file
//http://lab.polygonal.de/2007/05/10/bitwise-gems-fast-integer-math/; http://www.gamedev.net/topic/336890-custom-unsigned-int-64-bit-divide/
//extern int _fltused = 0;
#define USE_ARICODER
#include <MiniCRT.h>
#include <ModelBase.h>
#include <AdaptiveBitModel.h>
#include <BitModel.h>
#include <ByteBitModel.h>
#include <DoubleBitModel.h>
#include <HashBitModel.h>
#include <SSE.h>
#include <TextModel.h>
#include <ModelDefinitions.h>
#include <StreamCoder.h>
#ifndef USE_ARICODER
#include <RangeCoder32.h>
#else
#include <ArithmeticCoder.h>
#endif
#define DISK_BLOCK 65536
typedef struct FILE_BLOCK {
	PBUFFERED_FILE indata;
	PBUFFERED_FILE outdata;
} FILE_BLOCK, *PFILE_BLOCK;
BOOL OpenFiles(LPTSTR infile, LPTSTR outfile, PFILE_BLOCK files) {
	HANDLE hIn;
	HANDLE hOut;
	int tmpbyte = 0;
	BYTE lastbyte = 0;
	RemoveQuotes(infile);
	RemoveQuotes(outfile);
	if(infile[0] == ':') {
		hIn = stdin;
	} else {
		hIn = OpenFileForRead(infile);
	};
	if(hIn == INVALID_HANDLE) {
		ConsoleWriteLineError("Could not open input file.\0");
		return FALSE;
	};
	if(outfile[0] == ':') {
		hOut = stdout;
	} else {
		hOut = CreateFileForWrite(outfile);
	};
	if(hOut == INVALID_HANDLE) {
		CloseHandle(hIn);
		ConsoleWriteLineError("Could not create output file.\0");
		return FALSE;
	};
	files->indata = InitBufferedFile(hIn, DISK_BLOCK);
	files->outdata = InitBufferedFile(hOut, DISK_BLOCK);
	return TRUE;
};
void EncodeFile32(PBUFFERED_FILE inFile, PBUFFERED_FILE outFile, int mdl, int order) {
	BOOL bv;
	int cb;
	int i;
	PRangeCoder32 rc;
	PVOID bm;
	PStreamCoder sc;
	MODEL_SIG model = models[mdl];
	ConsoleWriteLineError("\0");
	ConsoleWriteError("Encoding using \0");
	ConsoleWriteError(model.mdlname);
	ConsoleWriteLineError("...\0");
	WriteBufferedByte(outFile, (BYTE) (order | (mdl << 5)));
	FlushBufferedFile(outFile, TRUE);
	rc = InitRC32(outFile);
	bm = model.initf(order);
	sc = InitStreamCoder(inFile);
	if(IsNull(bm)) {
		ConsoleWriteLineError("Out of memory.");
		return;
	};
	while(TRUE) {
		cb = SCReadByte(sc);
		if(cb == -1) {
			RC32Flush(rc);
			goto doneEncode;
		};
		for(i = 7; i >= 0; i--) {
			bv = ((cb >> i) & 1);
			RC32EncodeBit(rc, bv, model.predf(bm));
			model.feedf(bm, bv);
		};
	};
doneEncode:
	FlushBufferedFile(outFile, TRUE);
	CloseBufferedFile(inFile);
	CloseBufferedFile(outFile);
	model.freef(bm);
	DestroyStreamCoder(sc);
	DestroyRC32(rc);
};
void DoEncode(LPTSTR infile, LPTSTR outfile, int model, int order) {
	PFILE_BLOCK files = (PFILE_BLOCK) MemAlloc(sizeof(FILE_BLOCK) * 2);
	if(OpenFiles(infile, outfile, files) == TRUE) { EncodeFile32(files->indata, files->outdata, model, order); };
	MemFree(files);
};
void DecodeFile32(PBUFFERED_FILE inFile, PBUFFERED_FILE outFile) {
	int bv = 0;
	int vp = 0;
	int order = 0;
	BOOL bit = FALSE;
	PRangeCoder32 rc;
	PVOID bm;
	PStreamCoder sc;
	MODEL_SIG model;
	order = ReadBufferedByte(inFile);
	model = models[order >> 5];
	ConsoleWriteLineError("\0");
	ConsoleWriteError("Decoding using \0");
	ConsoleWriteError(model.mdlname);
	ConsoleWriteLineError("...\0");
	order = order & 0x1F;
	rc = InitRC32(inFile);
	bm = model.initf(order);
	sc = InitStreamCoder(outFile);
	if(IsNull(bm)) {
		ConsoleWriteLineError("Out of memory.");
		return;
	};
	while((rc->atEnd == FALSE) && (sc->done == FALSE)) {
		bit = RC32DecodeBit(rc, model.predf(bm));
		model.feedf(bm, bit);
		bv = bv | ((bit & 1) << (7 - vp));
		if(vp >= 7) {
			SCWriteByte(sc, (BYTE) bv);
			vp = 0;
			bv = 0;
		} else {
			vp++;
		};
	};
	FlushBufferedFile(outFile, TRUE);
	CloseBufferedFile(inFile);
	FlushBufferedFile(outFile, TRUE);
	CloseBufferedFile(outFile);
	model.freef(bm);
	DestroyStreamCoder(sc);
	DestroyRC32(rc);
};
void DoDecode(LPTSTR infile, LPTSTR outfile) {
	PFILE_BLOCK files = (PFILE_BLOCK) MemAlloc(sizeof(FILE_BLOCK));
	if(OpenFiles(infile, outfile, files) == TRUE) { DecodeFile32(files->indata, files->outdata); };
	MemFree(files);
};
void PrintHelp() {
	int i;
	ConsoleWriteLine("\0");
	ConsoleWriteLine("usage: RangeCoder c [order] [infile] <outfile>\0");
	ConsoleWriteLine("       RangeCoder c[model] [order] [infile] <outfile>\0");
	ConsoleWriteLine("       RangeCoder d [infile] <outfile>\0");
	ConsoleWriteLine("\0");
	ConsoleWriteLine("The model number can be one of the following:\0");
	for(i = 0; i < modelcount; i++) {
		ConsoleWrite("      \0");
		ConsoleWriteChar(dec[i]);
		ConsoleWrite(" - \0");
		ConsoleWrite(models[i].mdlname);
		if(i == 1) { ConsoleWrite(" (default)\0"); };
		ConsoleWriteLine("\0");
	};
	ConsoleWriteLine("\0");
	ConsoleWriteLine("note:  To use stdin/stdout, enter : as the filename\0");
};
void __cdecl main() {
	LPTSTR args[260];
	int argn = 0;
	int order = 0;
	int mdln = 0;
	InitStdio();
	args[0] = NULL;
	args[1] = NULL;
	args[2] = NULL;
	args[3] = NULL;
	args[4] = NULL;
	ConsoleWriteLineError("RangeCoder   V1.9                                  (c) 2011 by David Catt\0");
	argn = GetCommandLine(&args);
	InitModels();
	if(argn > 2) {
		if(*args[1] == 'c') {
			order = StringToInt(args[2]);
			if((order != -1) && (!IsNull(args[3]))) {
				order = order & 0x1F;
				if(IsNull(args[4]) == TRUE) { args[4] = AppendString(args[3], ".rc\0"); };
				if(args[1][1] != 0) { mdln = StringToInt(&args[1][1]); };
				if((mdln >= 0) && (mdln < modelcount)) { 
					DoEncode(args[3], args[4], mdln, order); 
				} else {
					DoEncode(args[3], args[4], 1, order);
				};
			} else {
				PrintHelp();
			};
		} else { if(*args[1] == 'd') {
			if(IsNull(args[3]) == TRUE) { args[3] = AppendString(args[2], ".dec\0"); };
			DoDecode(args[2], args[3]);
		} else {
			PrintHelp();
		}; };
	} else {
		PrintHelp();
	};
	ConsoleWriteLineError("\0");
	ExitProcess(0);
};