typedef struct SSEModel {
	int order;
	PValPair model;
	UINT mask;
	LONG context;
	ValPair temp;
	ValPair tmp;
} SSEModel, *PSSEModel;
PSSEModel InitSSE(int modelOrder) {
	LONG init;
	PSSEModel sse = (PSSEModel) MemAlloc(sizeof(SSEModel));
	sse->order = modelOrder;
	sse->mask = (1 << modelOrder) - 1;
	sse->context = 0;
	sse->model = (PValPair) MemAlloc(sizeof(ValPair) * (sse->mask + 1));
	if(IsInvalidPtr(sse->model)) { 
		MemFree(sse);
		return NULL;
	};
	for(init = 0; init <= sse->mask; init++) {
		sse->model[init].v0 = 1;
		sse->model[init].vt = 2;
	};
	return sse;
};
__forceinline UINT SSEPredict(PSSEModel sse, UINT prediction) {
	sse->context = (prediction >> (32 - sse->order)) & sse->mask;
	sse->tmp = sse->model[sse->context];
	return ((LONG) sse->tmp.v0 * (LONG) 0xFFFFFFFF) / ((LONG) sse->tmp.vt);
};
__forceinline void SSEUpdate(PSSEModel sse, BOOL bit) {
	sse->model[sse->context].vt++;
	if(bit == FALSE) { sse->model[sse->context].v0++; };
};
void DestroySSE(PSSEModel sse) {
	MemFree(sse->model);
	MemFree(sse);
};